@@ -20,6 +20,11 @@ from pyngrok import conf, ngrok
 import requests
 import shutil
 import questionary
+import boto3
+import time
+from collections import defaultdict
+from botocore.exceptions import ClientError, ParamValidationError
+from datetime import datetime
 from subprocess import PIPE
 
 def getVersion():
@@ -27,6 +32,10 @@ def getVersion():
 
 service_modules = ["ac-core", "ac-allowlist", "ac-automation", "ac-macro-security", "ac-migrations", "ac-proxy", "ac-utils"]
 
+CGREY    = '\33[90m'
+CGREEN    = '\033[92m'
+SLEEP = 5
+
 @click.command()
 @click.option('--verbose', '-v', is_flag=True, help="Verbose output")
 @click.option('--profile', '-p', help="AWS profile as the default environment")
@@ -47,9 +56,11 @@ service_modules = ["ac-core", "ac-allowlist", "ac-automation", "ac-macro-securit
 @click.option('--aws-home', '-aws-home', help="AWS home directory for config/credentials", default="~/.aws")
 @click.option('--disable-clone-prompt', '-disable-clone-prompt', help="Disable prompt for git clone.", default="True")
 @click.option('--version','-v', help="Specify App version", default="")
+@click.option('--filter','-f', help="Filter the list of AWS Log Groups, used as a wildcard match", default="")
+@click.option('--log-group','-n', help="AWS Log Group name", default="")
 @click.argument('command')
 @click.help_option("--help", "-h")
-def process(verbose, command, profile, env, stage, stack, app_suffix, build, api_domain, api_stage, app_api_domain, require_approval, tag, bitbucket_username, bitbucket_password, bitbucket_branch, bitbucket_tag_value, aws_home, disable_clone_prompt, version):
+def process(verbose, command, profile, env, stage, stack, app_suffix, build, api_domain, api_stage, app_api_domain, require_approval, tag, bitbucket_username, bitbucket_password, bitbucket_branch, bitbucket_tag_value, aws_home, disable_clone_prompt, version, filter, log_group):
     """
     Gathers information related to deployment and deploys the CDK stack
 
@@ -81,6 +92,8 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         Options:  --profile, --env, --stage, --stack, --api-domain, --api-stage, --app-api-domain
       run-pipeline  Run a Bitbucket pipeline for a service module
         Options:  --stack, --bitbucket-username, --bitbucket-password, --bitbucket-branch, --bitbucket-tag-value
+      tail-lg       Tails the logs for a AWS CloudWatch Logs group, use --filter to narrow down the list of Log Groups.
+        Options:  --profile, --filter, --log-group
       version       Print version information for the Appfire SDK
 
     Learn more - https://wiki.appfire.com/x/x6gjCQ
@@ -90,6 +103,8 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
     """
     log_level = logging.DEBUG if verbose else logging.INFO
     logging.basicConfig(level=log_level, stream=sys.stdout, format="%(message)s")
+    logging.getLogger('botocore').setLevel(logging.CRITICAL)
+    logging.getLogger('urllib3').setLevel(logging.CRITICAL)
     logger = logging.getLogger("ac_app_deploy")
     coloredlogs.install(level='DEBUG',logger=logger)
 
@@ -111,6 +126,10 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
     if command.lower() == 'docker-build':
         docker_build(logger,tag,disable_clone_prompt)
         return
+    if command.lower() == 'tail-lg':
+        filter = list(filter.split(","))
+        tail_log_group(profile,filter,log_group)
+        return
     
     logger.debug("checking environment " + env)
     if env == 'standalone':
@@ -663,6 +682,110 @@ def checkCDKEnvVariables():
     if os.getenv("AWS_ACCESS_KEY_ID") is None or os.getenv("AWS_SECRET_ACCESS_KEY") is None:
         print(f"AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables must be set.")
         sys.exit(1)
+
+
+def tail_log_group(profile,filter,log_group_name):
+    client = boto3.Session(profile_name=profile).client('logs')
+
+    # Get a list of log groups (use filter if set)
+    if (log_group_name == ""):
+
+        response = client.describe_log_groups()
+        names = []
+        log_groups = response['logGroups']
+
+        while 'nextToken' in response.keys():
+            response = client.describe_log_groups(nextToken=response['nextToken'])
+            log_groups.extend(response['logGroups'])
+
+        found = False
+        for log_group in log_groups:
+            log_group_name = log_group["logGroupName"]
+
+            for f in filter:
+                if f in log_group_name:
+                    print(log_group["logGroupName"])
+                    names.append(log_group_name)
+                    found = True
+
+        if not found:
+            print("No Log Group(s) found that match filter: ", filter)
+            return
+        else:
+            log_group = questionary.select(
+                "Log Group",
+                choices=names).ask()  # returns value of selection
+            checkForKeyboardInterrupt(log_group)
+    else:
+        log_group = log_group_name
+
+    filter_logs_events_kwargs = {
+            "logGroupName": log_group,
+            "interleaved": True,
+            "logStreamNamePrefix" : str(datetime.today().year)
+        }
+
+    return filter_log_events(filter_logs_events_kwargs)
+
+def filter_log_events(filter_logs_events_kwargs):
+    client = boto3.client('logs')
+    try:
+        for event in do_filter_log_events(client, filter_logs_events_kwargs):
+            #yield event
+            timestamp = event["timestamp"]
+            timestamp /= 1000
+            print(CGREEN + str(datetime.fromtimestamp(timestamp)), "  " +  CGREY + event["message"],end='')
+    except KeyboardInterrupt:
+        return
+
+def get_latest_events_and_timestamp(event_ids_per_timestamp):
+    if event_ids_per_timestamp:
+        # Keep only ids of the events with the newest timestamp
+        newest_timestamp = max(event_ids_per_timestamp.keys())
+        event_ids_per_timestamp = defaultdict(
+            set, {newest_timestamp: event_ids_per_timestamp[newest_timestamp]}
+        )
+    return event_ids_per_timestamp
+
+def reset_filter_log_events_params(fle_kwargs, event_ids_per_timestamp):
+    # Remove nextToken and update startTime for the next request
+    # with the timestamp of the newest event
+    if event_ids_per_timestamp:
+        fle_kwargs['startTime'] = max(
+            event_ids_per_timestamp.keys()
+        )
+    fle_kwargs.pop('nextToken', None)
+
+def do_filter_log_events(client, filter_logs_events_kwargs):
+    SLEEP = 5
+    event_ids_per_timestamp = defaultdict(set)
+    while True:
+        try:
+            response = client.filter_log_events(
+                **filter_logs_events_kwargs)
+        except (ClientError, ParamValidationError):
+            sys.exit("Invalid MFA one time pass code")
+
+        for event in response['events']:
+            # For the case where we've hit the last page, we will be
+            # reusing the newest timestamp of the received events to keep polling.
+            # This means it is possible that duplicate log events with same timestamp
+            # are returned back which we do not want to yield again.
+            # We only want to yield log events that we have not seen.
+            if event['eventId'] not in event_ids_per_timestamp[event['timestamp']]:
+                event_ids_per_timestamp[event['timestamp']].add(event['eventId'])
+                yield event
+        event_ids_per_timestamp = get_latest_events_and_timestamp(
+            event_ids_per_timestamp
+        )
+        if 'nextToken' in response:
+            filter_logs_events_kwargs['nextToken'] = response['nextToken']
+        else:
+            reset_filter_log_events_params(
+                filter_logs_events_kwargs,
+                event_ids_per_timestamp
+            )
+            time.sleep(SLEEP)
 class ShellCommandFailed(Exception):
     """ Executing a shell command failed """
 
